@@ -3,31 +3,35 @@
 
 # (c)Ing. Zdenek Dlauhy, Michal Dlauhý, pripravto.cz
 
-class log(object):
 
-    @staticmethod
-    def debug(msg):
-        print(msg)
-    info = debug
-    warn = debug
-    error = debug
+import subprocess
+import os
 
+# TODO: fix logging and messages
 # import logging
 # logging.basicConfig(level=logging.DEBUG)
 # log = logging.getLogger("ans wrapper")
 
-import subprocess
+class log(object):
+
+    @staticmethod
+    def info(msg):
+        print(msg)
+
+    @staticmethod
+    def debug(msg):
+        pass
+    warn = info
+    error = info
 
-import os
 try:
     import ansible.runner
     import ansible.inventory
-except:
-    log.error("Ansible is not installed! Ansiblator will not work. ")
+except ImportError as e:
+    log.error("Cannot import Ansible! Ansiblator will not work. {}".format(e))
     log.error("Run:  pip install ansible")
 
 
-
 """
 Main module of Ansiblator - wrapper around Ansible for Python
 
@@ -37,13 +41,6 @@ Classes
 Ansiblator() - actuall wrapper
 DictToInventory - convertor for inventory files
 
-TODO
-------
-- improve fabric like api for more improvments
-- make more tests
-- try to improve dynamic inventory
-- improve logging
-- improve DictToInventory mapper, so more options are possible
 
 """
 try:
@@ -158,8 +155,8 @@ class Ansiblator(object):
 
     code::
 
-        from ansiblator.api import Ansiblator
-        ans = Ansiblator()
+        import ansiblator.api as an
+        ans = an.Ansiblator()
         ret = ans.local("uname -a", now=True, use_shell=True)
         ans.run("uname -a", now=True)
         ans.runner("uptime")
@@ -168,22 +165,36 @@ class Ansiblator(object):
 
     specify ansible hosts file and select pattern::
 
-        ans = Ansiblator(inventory="/tmp/ansible_file", pattern="pc")
+        ans = an.Ansiblator(inventory="/tmp/ansible_file", pattern="pc")
 
     use dictionary to create inventory::
 
         inv = {'pc':[{'ssh_host':'192.168.0.10', 'ssh_user':'test_user', 'su_user':'root'},
                     {'ssh_host':'192.168.0.12', 'ssh_user':'test_user2', 'su_pass':'paasswd','su_user':'root'}]}
-        ans = Ansiblator(inventory=inv)
+        ans = an.Ansiblator(inventory=inv)
         ans.run("uname -a", now=True)
 
     prepare commands and run after::
 
-        ans = Ansiblator(run_at_once=False)
+        ans = an.Ansiblator(run_at_once=False)
         ans.get(src="/tmp/file", dest="/tmp/")
         ans.get(src="/tmp/file2", dest="/tmp/")
         ans.run_all()
 
+    make custom class::
+
+        class Automatization(Ansiblator):
+
+            def update_server(self, su=True,sudo=False):
+                self.run("apt-get update", su=su, sudo=sudo)
+                self.run("apt-get upgrade -y", su=su, sudo=sudo)
+
+    use class and more patterns::
+
+        ans = Automatization(pattern=['servers', 'production', 'test', 'pc'])
+        ans.update_server()
+
+
     With this, you can create full commands or functions and just pass to them
     pattern and run at the end.
 
@@ -207,13 +218,17 @@ class Ansiblator(object):
     TODO
     ====
 
-    - improve fabric-like api for more improvments
     - make more tests
-    - try to improve dynamic inventory attachments
     - make automatic translating of methods to ansible modules
     - improve logging
     - improve DictToInventory mapper, so more options are possible, such as groups and so on
 
+    CHANGES
+    =======
+
+    - ability to run on more patterns
+    - fixes on more runs
+
     Info
     ====
     For more information you can consult functions or actual Ansible documentation.
@@ -257,7 +272,7 @@ class Ansiblator(object):
         self.stop_on_error = stop_on_error
         self.use_shell = use_shell
         self.inventory = self.parse_inventory(inventory)
-        log.debug("Intializing ansible: {}".format(self))
+        log.info("Intializing ansible: {}".format(self))
 
     def get_host_var(self, host):
         return self.inventory.get_variables(host)
@@ -270,17 +285,18 @@ class Ansiblator(object):
         home = os.environ.get('HOME', '/tmp/')
         if isinstance(inventory, str):
             if not inventory.startswith("/"):
-                inventory = "{}/{}".format(home, inventory)
+                filename = "{}/{}".format(home, inventory)
+            else:
+                filename = "{}".format(inventory)
         elif isinstance(inventory, dict):
             filename = "{}/{}".format(home, "ansible_hosts")
             DictToInventory(inventory, filename=filename)
-            inventory = filename
         else:
             pass
         if isinstance(inventory, ansible.inventory.Inventory):
             pass
         else:
-            inventory = ansible.inventory.Inventory(inventory)
+            inventory = ansible.inventory.Inventory(filename)
         return inventory
 
 
@@ -331,6 +347,8 @@ class Ansiblator(object):
             module_name = self.module_name
         if pattern is None:
             pattern = self.pattern
+        if not isinstance(pattern, list):
+            pattern = [pattern]
         if inventory is None:
             inventory = self.inventory
         else:
@@ -347,25 +365,40 @@ class Ansiblator(object):
         if host is None:
             host = self.host
         if host is None:
-            data = self.get_group_host(pattern)
-            host_list = None
+            for pat in pattern:
+                data = self.get_group_host(pat)
+                host_list = None
+                info = dict(locals())
+                del info['self']
+                runner = self._runner_ans(**info)
+                self._run(runner, now)
         else:
             host_list = [host]
             pattern = None
             data = self.get_host_var(host)
+            info = dict(locals())
+            del info['self']
+            runner = self._runner_ans(**info)
+            self._run(runner, now)
+
+    def _runner_ans(self, data=None, cmd=None, pattern=None, module_name=None, inventory=None,
+                   now=None, su=None, sudo=None, su_user=None, su_pass=None,
+                   sudo_pass=None, host_list=None, host=None, pat=None, **kwargs):
         su_pass, sudo_pass, su_user = self.get_host_vars(data, su, sudo, su_pass, sudo_pass, su_user)
         info = dict(locals())
         del info['data']  # remove to skip output of password out to shell
         del info['su_pass']
         del info['sudo_pass']
-        log.info(info)
-        cmd = self.create_argument(cmd, kwargs)
+        del info['pattern']
+        log.debug(info)
+        cmd = self.create_argument(cmd, kwargs.get('kwargs', {}))
         log.info("Creating new command '{}' on '{}'".format(cmd, module_name))
-        runner = ansible.runner.Runner(pattern=pattern, host_list=host_list, module_name=module_name,
+        runner = ansible.runner.Runner(pattern=pat, host_list=host_list, module_name=module_name,
                                        module_args=cmd, inventory=inventory,
                                        su=su, sudo=sudo, su_user=su_user, su_pass=su_pass,
                                        sudo_pass=sudo_pass)
-        return self._run(runner, now)
+        return runner
+
 
     def runner_local(self, cmd="uptime", module_name=None, now=None, use_shell=None, **kwargs):
         """
@@ -390,7 +423,6 @@ class Ansiblator(object):
         :param use_shell: use shell when subbprocess is running
         :type use_shell: bool
 
-
         function to run or append runner
         """
         if now is None:
@@ -400,21 +432,7 @@ class Ansiblator(object):
         if now:
             log.info("Running {}".format(runner))
             if isinstance(runner, dict):
-                try:
-                    ret_code = subprocess.Popen(runner['cmd'], stdout=subprocess.PIPE, shell=use_shell)
-                    out, err = ret_code.communicate()
-                    return_c = ret_code.returncode
-                except OSError as e:
-                    log.error(e)
-                    return_c = 1
-                    out = None
-                    err = e
-
-                if return_c == 0:
-                    failed = False
-                else:
-                    failed = True
-                ret_code = {'contacted':{'local':{'stdout':out, 'failed':failed , 'stderr':err, 'ret_code':return_c}}, 'dark':{}}
+                ret_code = self._runner_local(runner, use_shell)
             else:
                 ret_code = runner.run()
             self.parse_ret_code(ret_code)
@@ -425,6 +443,23 @@ class Ansiblator(object):
             self.runners.append(runner)
             return {"run":False}
 
+    def _runner_local(self, runner, use_shell):
+        """low level local runner"""
+        try:
+            ret_code = subprocess.Popen(runner['cmd'], stdout=subprocess.PIPE, shell=use_shell)
+            out, err = ret_code.communicate()
+            return_c = ret_code.returncode
+        except OSError as e:
+            log.error(e)
+            return_c = 1
+            out = None
+            err = e
+        if return_c == 0:
+            failed = False
+        else:
+            failed = True
+        return {'contacted':{'local':{'stdout':out, 'failed':failed , 'stderr':err, 'ret_code':return_c}}, 'dark':{}}
+
 
     def runner(self, cmd="uptime", use_ansible=True, module_name="command", inventory=None, now=None, pattern=None, **kwargs):
         """
@@ -443,7 +478,7 @@ class Ansiblator(object):
 
         main runner method which acts as medium to other calls
         """
-        log.debug("Making new runner: {}".format(cmd))
+        log.info("Making new runner: {}".format(cmd))
         if use_ansible:
             return self.runner_ans(cmd=cmd, module_name=module_name, inventory=inventory, now=now, pattern=pattern, **kwargs)
         else:
